import data, pygame
from pygsear.Drawable import *
from pygsear.Util import load_image

from utils import *

class Moveable(Image):
    dest=None
    moving=0
    offset=24
    vel=(0,0)
    speed=2
    def __init__(self, name="flower", coords=(0,0)):
        Image.__init__(self,filename=DCT_TILE[name])
        self.coords=coords
        self.set_position(coord2pos(coords))
        self.lastpos=self.get_position()
        #self.prepare()

    def hide(self,bol):
        if bol:
            self.lastpos=self.get_position()
            self.set_position((-64,-64))
        else:
            self.set_position(self.lastpos)

    def get_fwd(self, looks):
        x,y = self.coords
        if looks=="up":
            y-=1
        elif looks=="down":
            y+=1
        elif looks=="left":
            x-=1
        elif looks=="right":
            x+=1
        return (x,y)

    def move(self):
        if self.dest:
            self.set_newpos()
        Image.move(self)

    def moveto(self,looks):
        vx,vy=(0,0)
        if looks=="up":
            vy=-self.speed
        elif looks=="down":
            vy=self.speed
        elif looks=="left":
            vx=-self.speed
        elif looks=="right":
            vx=self.speed
        self.vel=(vx,vy)
        self.coords=self.get_fwd(looks)
        self.dest=coord2pos(self.coords)
        self.moving=1

    def set_newpos(self):
        pos=self.get_position()
        if self.moving:
            self.set_position((pos[0]+self.vel[0],pos[1]+self.vel[1]))
            pos=self.get_position()
        if not (pos[0]-self.offset)%32 and not (pos[1]-self.offset)%32:
            #self.set_position(self.dest)
            self.dest=None
            self.vel=(0,0)
            self.moving=0

    def bgprepare(self):
        self.bg = pygame.Surface(self.image.get_size())
        self.bg.blit(self.window.bg, (0, 0), self.rect)

    def bgdraw(self):
        rect = self.rect
        self.window.bg.blit(self.image, rect)
        self.window.screen.blit(self.image, rect)
        pygame.display.update(rect)

    def bgclear(self):
        w, h = self.rect.size
        r = pygame.Rect(0, 0, w, h)
        self.window.bg.blit(self.bg, self.rect, r)
        self.window.screen.blit(self.bg, self.rect, r)
        pygame.display.update(self.rect)

class AnimatedStationary(Image):
    def __init__(self,files=None,coords=None,ft=250):
        self.imageNum=0
        self.ticks = 250
        self.flipticks = 250
        self.images = []
        for i in files:
            self.images.append(load_image(i))
        Image.__init__(self, image=self.images[0])
        self.flipticks=ft
        self.coords=coords
        if self.coords:
            self.set_position(coord2pos(coords))
        self.lastpos=self.get_position()

    def hide(self,bol):
        if bol:
            self.lastpos=self.get_position()
            self.set_position((-64,-64))
        else:
            self.set_position(self.lastpos)

    def flip(self):
        """switch to next image in sequence"""
        self.image = self.images[self.imageNum]
        self.imageNum += 1
        if self.imageNum >= len(self.images):
            self.imageNum = 0
        self.set_size(self.image.get_size())
        self.set_crect()

    def move(self):
        """set position to next position on path and flip to next image"""
        self.ticks -= conf.ticks
        if self.ticks < 0:
            self.flip()
            self.ticks += self.flipticks
        Image.move(self)


class Tile(Stationary):
    '''basic tile, can switch image
    '''
    def __init__(self, char="G", coords=(0,0)):
        self.kind=DCT_CHAR2NAME[char]
        self.coords=coords
        Stationary.__init__(self,sprite=Image(filename=DCT_TILE[self.kind]))
        self.set_position(coord2pos(coords))

    def switch_kind(self,kind):
        self.kind=kind

    def switch_sprite(self,sprite):
        pos=self.get_position()
        self.sprite.clear()
        self.sprite=sprite
        self.set_position(pos)


class Level():
    ''' basic Level class.. well, it used to be..
    '''
    def __init__(self, nr=0):
        self.map={}
        self.oldmap={}
        self.size=(24,17)
        self.start=(0,0)
        self.start2=(0,0)
        self.exit=None
        self.airlock=None
        self.trigger=None
        self.flowers=[]
        self.crates=[]
        self.animated=[]
        self.pellets=0
        self.crateplaces=[]
        f=LVL[nr]
        f=open(f,"r")
        x,y=(0,0)
        for line in f:
            x=0
            for char in line.strip():
                self.oldmap[(x,y)]=char
                if char=="S":
                    self.start=(x,y)
                elif char=="E":
                    self.exit = (x,y)
                elif char=="A":
                    self.airlock=(x,y)
                elif char=="T":
                    self.trigger=(x,y)
                elif char=="P":
                    self.pellets+=1
                elif char=="F":
                    char="G"
                    self.flowers.append(Moveable(coords=(x,y)))
                elif char=="C":
                    char="G"
                    self.crates.append(Moveable(name="crate",coords=(x,y)))
                elif char=="X":
                    files=[DCT_TILE["cratetarget1"],DCT_TILE["cratetarget2"]]
                    a = AnimatedStationary(files=files,coords=(x,y),ft=1000)
                    self.animated.append(a)
                if char!=".":
                    self.map[(x,y)]=Tile(char,(x,y))
                x+=1
            y+=1
        f.close()
        self.load_walls()
        self.find_start2()
        for i in self.animated:
            i.udraw()


    def find_start2(self):
        x,y=self.start
        pos=[(x+1,y),(x-1,y),(x,y+1),(x,y-1)]
        for i in pos:
            if self.map.has_key(i):
                if not LST_BLOCKING.__contains__(self.map[i].kind):
                    self.start2=i


    def rem_pellet(self):
        self.pellets-=1

    def redraw(self):
        charmap=self.oldmap
        lvlmap=self.map
        self.pellets=0
        crateplaces=[]
        flowerplaces=[]
        repaint=["G","K","B","D","A","E","X","T","S","P"]
        for i in charmap.keys():
            c = charmap[i]
            if repaint.__contains__(c):
                self.map[i]=Tile(c,i)
            if c=="F":
                flowerplaces.append(i)
                self.map[i]=Tile("G",i)
            elif c=="C":
                crateplaces.append(i)
                self.map[i]=Tile("G",i)
            elif c=="P":
                self.pellets+=1
        for i in self.crates:
            pos=crateplaces.pop()
            i.coords=pos
            i.set_position(coord2pos(pos))
        for i in self.flowers:
            pos=flowerplaces.pop()
            i.coords=pos
            i.set_position(coord2pos(pos))


    def find_asset(self, asset="crates", coords=(0,0)):
        if asset=="crates":
            lst = self.crates
        elif asset=="flowers":
            lst = self.flowers
        else:
            return 0

        for i in lst:
            if coords==i.coords:
                return i
        return 0

    def load_walls(self):
        """This is painful. Why did I do that?"""
        lvl=["airlockc","airlocko","crate","cratetarget1","cratetarget2",
         "doorc","dooro","exit","flower","ground","key","pellet","start",
         "trigger1","trigger2","trigger"]
        m=self.map

        for i in m.keys():
            x,y = i
            up = (x,y-1)
            down = (x,y+1)
            left = (x-1,y)
            right = (x+1,y)
            tile=m[i]
            if tile.kind=="wall":
                #check which tiles AROUND tile are lvl tiles
                upper=0
                lower=0
                lefter=0
                righter=0
                if m.has_key(up) and lvl.__contains__(m[up].kind):
                    upper=1
                if m.has_key(down) and lvl.__contains__(m[down].kind):
                    lower=1
                if m.has_key(left) and lvl.__contains__(m[left].kind):
                    lefter=1
                if m.has_key(right) and lvl.__contains__(m[right].kind):
                    righter=1

                test = [upper,lower,lefter,righter]
                if test==[0,1,0,0]:
                    tile.switch_sprite(Image(filename=DCT_TILE["wall1"]))
                if test==[1,0,0,0]:
                    img=pygame.image.load(DCT_TILE["wall1"])
                    img=pygame.transform.rotate(img,180)
                    tile.switch_sprite(Image(image=img))
                if test==[0,0,1,0]:
                    img=pygame.image.load(DCT_TILE["wall1"])
                    img=pygame.transform.rotate(img,-90)
                    tile.switch_sprite(Image(image=img))
                if test==[0,0,0,1]:
                    img=pygame.image.load(DCT_TILE["wall1"])
                    img=pygame.transform.rotate(img,90)
                    tile.switch_sprite(Image(image=img))

                if test==[0,1,0,1]:
                    tile.switch_sprite(Image(filename=DCT_TILE["wall2"]))
                if test==[1,0,1,0]:
                    img=pygame.image.load(DCT_TILE["wall2"])
                    img=pygame.transform.rotate(img,180)
                    tile.switch_sprite(Image(image=img))
                if test==[1,0,0,1]:
                    img=pygame.image.load(DCT_TILE["wall2"])
                    img=pygame.transform.rotate(img,90)
                    tile.switch_sprite(Image(image=img))
                if test==[0,1,1,0]:
                    img=pygame.image.load(DCT_TILE["wall2"])
                    img=pygame.transform.rotate(img,-90)
                    tile.switch_sprite(Image(image=img))

                if test==[1,1,0,0]:
                    tile.switch_sprite(Image(filename=DCT_TILE["wall2_2"]))
                if test==[0,0,1,1]:
                    img=pygame.image.load(DCT_TILE["wall2_2"])
                    img=pygame.transform.rotate(img,90)
                    tile.switch_sprite(Image(image=img))

                if test==[0,1,1,1]:
                    tile.switch_sprite(Image(filename=DCT_TILE["wall3"]))
                if test==[1,0,1,1]:
                    img=pygame.image.load(DCT_TILE["wall3"])
                    img=pygame.transform.rotate(img,180)
                    tile.switch_sprite(Image(image=img))
                if test==[1,1,0,1]:
                    img=pygame.image.load(DCT_TILE["wall3"])
                    img=pygame.transform.rotate(img,90)
                    tile.switch_sprite(Image(image=img))
                if test==[1,1,1,0]:
                    img=pygame.image.load(DCT_TILE["wall3"])
                    img=pygame.transform.rotate(img,-90)
                    tile.switch_sprite(Image(image=img))

                if test==[1,1,1,1]:
                    tile.switch_sprite(Image(filename=DCT_TILE["wall4"]))



if __name__=="__main__":
    import time
    l=Level()
    print "Start ", l.start
    l.redraw()

    time.sleep(100)

