import pygame, random
from pygsear import conf
from pygsear.Widget import *
from pygsear.Drawable import *
from pygsear.Util import *
from pygsear.Event import *

from utils import *

class CatButton(StationaryButton):
    def __init__(self, filename=None, pos=(0,768),callback=None):
        sprite=Image(filename=filename)
        group=None
        Stationary.__init__(self, w=None, sprite=sprite)
        SpriteButton.__init__(self, sprite, callback, group)
        self.image = sprite.image
        self.rect = sprite.rect
        self.set_position(pos)


class Num(Stationary): #page numbers
    '''basic number image, can switch image
    '''
    def __init__(self, w=None, filename=None, pos=(-200,-200)):
        Stationary.__init__(self,sprite=Image(filename=filename))
        self.set_position(pos)

    def switch_sprite(self,sprite):
        pos=self.get_position()
        self.sprite=sprite
        self.set_position(pos)

class Errorlog(Num):
    def __init__(self, pos=(840,600)):
        self.rect=pygame.Rect(pos,(170,65))
        #msg="Error: Page 1 Line 2 Error: Page 1 Line 20 Error: Page 10 Line 2"
        self.msg=""
        self.lines=0
        self.full=False
        text=render_textrect(string=self.msg,
                            rect=self.rect,
                            font="Courier-Bold",
                            text_color=(255,255,255),
                            bgcolor=(0,0,0),
                            fontSize=12)
        Stationary.__init__(self,sprite=Image(image=text))
        self.set_position(pos)

    def reinit(self):
        self.full=False
        self.lines=0
        self.msg=""
        self.gen_image("")

    def switch_sprite(self,sprite):
        pos=self.get_position()
        self.sprite=sprite
        self.set_position(pos)

    def gen_image(self, msg):
        if not self.full:
            try:
                self.msg = self.msg + msg
                test=render_textrect(string=self.msg,
                            rect=self.rect,
                            font="Courier-Bold",
                            text_color=(255,255,255),
                            bgcolor=(0,0,0),
                            fontSize=12)
                self.switch_sprite(Image(image=test))
                self.lines+=1
            except TextRectException:
                print "Could not render Errorlog. Too much text!"
        if self.lines > 3:
            self.full=True

class Messenger(Num):
    def __init__(self, rect=((835,23),(162,162))):
        self.rect=pygame.Rect(rect)
        self.msg=""
        self.bgcolor=(0,0,0)
        text=render_textrect(string=self.msg,
                            rect=self.rect,
                            font="Courier-Bold",
                            text_color=(255,255,255),
                            bgcolor=self.bgcolor,
                            fontSize=12)
        Stationary.__init__(self,sprite=Image(image=text))
        self.set_position(rect[0])

    def switch_sprite(self,sprite):
        pos=self.get_position()
        self.sprite=sprite
        self.set_position(pos)

    def set_recty(self, rect):
        self.rect=pygame.Rect(rect)

    def gen_image(self, msg):
        try:
            #self.clear()
            self.msg=msg
            test=render_textrect(string=self.msg,
                        rect=self.rect,
                        font="Courier-Bold",
                        text_color=(255,255,255),
                        bgcolor=self.bgcolor,
                        fontSize=12)
            self.switch_sprite(Image(image=test))
        except TextRectException:
            print "Could not render level message. Too much text!"


class HNum(Num):  #the highlighted line numbers
    pos=[(11,647),(11,666),(11,685),(11,704),(11,723),
        (208,647),(208,666),(208,685),(208,704),(208,723),
        (405,647),(405,666),(405,685),(405,704),(405,723),
        (604,647),(604,666),(604,685),(604,704),(604,723)]
    def highlight(self, nr):
        if nr >= 0 and nr < len(self.pos):
            pos=self.pos[nr]
            self.sprite=Image(filename=LNUM[nr])
            self.set_position(pos)
        else:
            self.set_position((-200,-200))

class FuncButton(StationaryButton):
    kwargs=None
    sprites=[]
    def __init__(self,name=None, callback=None, group=None):
        StationaryButton.__init__(self,sprite=Image(filename=DCT_INT[name]),
                                    callback=callback,
                                    group=group)
        if name=="empty":
            sprite2=Image(filename=DCT_INT[name])
        else:
            sprite2=Image(filename=DCT_INT[name+"_2"])
        self.sprites=[self.sprite, sprite2]
        self.lastpos=self.get_position()

    def hide(self,bol):
        if bol:
            self.lastpos=self.get_position()
            self.set_position((-64,-64))
        else:
            self.set_position(self.lastpos)

    def fire(self, pygameEvent):
        self.armed = 0
        self.callback(pygameEvent, self.kwargs)

    def released(self, pygameEvent):
        pos = pygameEvent.pos
        try:
            offset = self.window.rect[0:2]
        except AttributeError:
            offset = (0, 0)
        if self.rect.move(offset[0],offset[1]).collidepoint(pos) \
          and self.armed:
            self.fire(pygameEvent)
            self.switch_images()
        else:
            self.armed = 0

    def arm(self):
        self.armed = 1
        self.switch_images()

    def set_kwargs(self, kwargs):
        self.kwargs=kwargs

    def switch_images(self):
        if self.armed:
            pos=self.get_position()
            self.sprite = self.sprites[1]
            self.set_position(pos)
        else:
            pos=self.get_position()
            self.sprite = self.sprites[0]
            self.set_position(pos)
    def highlight(self, nr):
        pos=self.get_position()
        self.sprite = self.sprites[nr]
        self.set_position(pos)

class MacroButton(FuncButton):
    kwargs=None
    sprites=[]

    def fire(self, pygameEvent):
        self.armed = 0
        if pygameEvent.button==1:
            self.callback(pygameEvent, self.kwargs)
        elif pygameEvent.button==3:
            self.callback2(pygameEvent, self.kwargs2)

    def released(self, pygameEvent):
        pos = pygameEvent.pos
        try:
            offset = self.window.rect[0:2]
        except AttributeError:
            offset = (0, 0)
        if self.rect.move(offset[0],offset[1]).collidepoint(pos) \
          and self.armed:
            self.armed=0
            if pygameEvent.button==1:
                self.switch_images()
            self.fire(pygameEvent)
        else:
            self.armed = 0

    def arm(self):
        self.armed=1

    def clicked(self, pygameEvent):
        pos = pygameEvent.pos
        try:
            offset = self.window.rect[0:2]
            #print 'off', offset
        except AttributeError:
            offset = (0, 0)

        if self.rect.move(offset[0], offset[1]).collidepoint(pos):
            self.arm()
            if pygameEvent.button==1:
                self.switch_images()
            return 1
        else:
            return 0


    def set_callback2(self, callback2, kwargs2):
        self.events.add(MOUSEBUTTONDOWN_Event(button=3,callback=self.clicked))
        self.events.add(MOUSEBUTTONUP_Event(button=3,callback=self.released))
        self.callback2=callback2
        self.kwargs2=kwargs2

class QButton(FuncButton):
    def released(self, pygameEvent):
        pos = pygameEvent.pos
        try:
            offset = self.window.rect[0:2]
        except AttributeError:
            offset = (0, 0)
        if self.rect.move(offset[0],offset[1]).collidepoint(pos) \
          and self.armed:
            self.fire(pygameEvent)
        else:
            self.armed = 0

    def switch_images(self):
        if self.armed:
            pos=self.get_position()
            #self.sprite = self.sprites[1]
            #self.set_position(pos)

class EditButton(FuncButton):
    def moveup(self, pos):
        self.kwargs[0] -= 1
    def movedown(self, pos):
        self.kwargs[1] += 1

class Line(FuncButton):
    command = "empty"
    def set_command(self, cmd):
        self.command = cmd
    def moveTo(self, pos):
        self.sprite.set_position(pos)
        self.draw()

    def highlight(self, high):
        pos=self.get_position()
        if high:
            self.sprite = self.sprites[1]
        else:
            self.sprite = self.sprites[0]
        self.set_position(pos)

    def switch_images(self):
        pass

    def set_img(self,name="empty"):
        pos=self.get_position()
        if name!="empty":
            sprite=Image(filename=DCT_INT[name])
            sprite2=Image(filename=DCT_INT[name+"_2"])
        else:
            sprite=Image(filename=DCT_INT[name])
            sprite2=Image(filename=DCT_INT[name])
        self.sprite=sprite
        self.sprites=[sprite,sprite2]
        self.set_position(pos)

class CatBot(Image):
    def __init__(self,name):
        self.looks = "down"
        self.imageNum=0
        self.ticks = 250
        self.start=(-1,-1)
        self.coords=(0,0)
        self.flipticks=250
        self.dest=()
        self.moving=0
        self.speed=2
        self.vel=(0,0)
        self.offset=24
        self.active=0
        self.keys=0
        self.go_fwd=0
        path=DCT_CHARAS[name]+"/"+name+"_"
        face=["down","right","left","up"]
        self.dct_faces={}
        for i in face:
                facelist=[]
                self.dct_faces[i]=[]
                for j in range(4):
                    file = path + i + str(j+1) + ".png"
                    self.dct_faces[i].append(load_image(filename=file))
        self.image=self.dct_faces["down"][0]
        self.images=self.dct_faces["down"]
        Image.__init__(self, filename=path+"down1.png")
        self.set_position((-300,-300))
        self.lastpos=(-300,-300)

    def activate(self,coords):
        self.looks="down"
        self.active=1
        self.start=coords
        self.set_position(coord2pos(coords))
        self.coords=coords

    def hide(self,bol):
        if bol:
            self.lastpos=self.get_position()
            self.set_position((-64,-64))
        else:
            self.set_position(self.lastpos)

    def reinit(self, catbot=0):
        self.looks="down"
        self.images=self.dct_faces["down"]
        self.image=self.dct_faces["down"][0]
        self.keys=0
        self.active=0
        self.moving=0
        if catbot:
            self.set_position((-64,-164))
        else:
            self.set_position((-64,-64))
        self.coords=((-2,-2))
        self.dest=()
        self.vel=(0,0)

    def flip(self):
        """switch to next image in sequence"""
        self.image = self.images[self.imageNum]
        self.imageNum += 1
        if self.imageNum >= len(self.images):
            self.imageNum = 0
        self.set_size(self.image.get_size())
        self.set_crect()

    def move(self):
        """set position to next position on path and flip to next image"""
        self.ticks -= conf.ticks
        if self.ticks < 0:
            self.flip()
            self.ticks += self.flipticks

        if self.dest:
            self.set_newpos()
        Image.move(self)

    def set_newpos(self):
        pos=self.get_position()
        if self.moving:
            self.set_position((pos[0]+self.vel[0],pos[1]+self.vel[1]))
            pos=self.get_position()
        if not (pos[0]-self.offset)%32 and not (pos[1]-self.offset)%32:
            self.set_position(self.dest)
            self.dest=()
            self.vel=(0,0)
            self.moving=0

    def turn(self, dir):
        if dir=="l":
            face=LEFT[self.looks]
        else:
            face=RIGHT[self.looks]
        pos=self.get_position()
        self.looks=face
        self.images=self.dct_faces[face]
        self.image=self.images[0]
        self.set_position(pos)

    def get_fwd(self):
        x,y = self.coords
        if self.looks=="up":
            y-=1
        elif self.looks=="down":
            y+=1
        elif self.looks=="left":
            x-=1
        elif self.looks=="right":
            x+=1
        return (x,y)

    def forward(self):
        self.moving=1
        x,y=self.get_position()
        vx,vy=(0,0)
        if self.looks=="up":
            y-=32
            vy=-self.speed
        elif self.looks=="down":
            y+=32
            vy=self.speed
        elif self.looks=="left":
            x-=32
            vx=-self.speed
        elif self.looks=="right":
            x+=32
            vx=self.speed
        self.coords=self.get_fwd()
        self.vel=(vx,vy)
        self.dest=(x,y)

    def add_key(self, num):
        self.keys += num

    def do(self, cmd):
        if cmd=="left":
            self.turn("l")
        elif cmd=="right":
            self.turn("r")
        elif cmd=="fwd":
            self.forward()
        elif cmd=="wait":
            pass
        elif cmd=="push":
            self.forward()
        elif cmd=="pickup":
            pass
        elif cmd=="use":
            pass

    def set_coords(self,coords):
        self.coords=coords
        self.set_position(coord2pos(coords))

if __name__=="__main__":
    from pygsear.Game import Game
    g=Game()
    c=CatBot()
    g.sprites.add(c)
    g.mainloop()
